# -*- coding: utf-8 -*-
import json
from .wedpr_user_information import WeDPRUserInformation


class WeDPRTokenContent:
    USER_TOKEN_CLAIM = "user"

    def __init__(self, token_content):
        self._token_content = token_content

    def get_token_content(self):
        return self._token_content

    def set_token_content(self, _token_content):
        self._token_content = _token_content

    def get_token_content_by_key(self, key):
        if key in self._token_content:
            return self._token_content[key]
        return None

    def get_user_information(self) -> WeDPRUserInformation:
        user_info = self.get_token_content_by_key(
            WeDPRTokenContent.USER_TOKEN_CLAIM)
        if user_info is None:
            return None
        return WeDPRUserInformation.decode(user_info)

    @staticmethod
    def deserialize(token_payload):
        return WeDPRTokenContent(token_payload)



# -*- coding: utf-8 -*-
import json
from typing import Any


class BaseObject:
    def set_params(self, **params: Any):
        for key, value in params.items():
            setattr(self, key, value)
            if hasattr(self, f"{key}"):
                setattr(self, f"{key}", value)
        return self


class WeDPRUserInformation(BaseObject):
    def __init__(self, user_name=None, role_name=None, **params):
        self.username = user_name
        self.roleName = role_name
        self.set_params(**params)

    @staticmethod
    def decode(json_str):
        if json_str is None or len(json_str) == 0:
            return None
        user_info_dict = json.loads(json_str)
        return WeDPRUserInformation(**user_info_dict)



"""
A JupyterHub authenticator class for use with WeDPR as an identity provider.
"""
from jupyter_server.auth.identity import IdentityProvider
from jupyter_server.auth.identity import User
import jwt
from .wedpr_token_content import WeDPRTokenContent
from tornado import web
from traitlets import Unicode, default
import psutil
import os


class WeDPRIdentityProvider(IdentityProvider):
    """Authenticate local UNIX users with WeDPR-Auth"""
    AUTH_TOKEN_FIELD = "Authorization"
    AUTH_ALGORITHMS = ["HS256", "HS512", "HS384"]

    auth_secret = Unicode("<generated>",
                          help="auth_secret").tag(config=True)

    CURRENT_USER = psutil.Process().username()

    @default("auth_secret")
    def _auth_secret_default(self):
        if os.getenv("JUPYTER_AUTH_SECRET"):
            return os.getenv("JUPYTER_AUTH_SECRET")
        return None

    def _get_token_from_header(self, handler: web.RequestHandler):
        try:
            token = handler.request.headers.get(
                WeDPRIdentityProvider.AUTH_TOKEN_FIELD, "")
            self.log.info(f"#### _get_token_from_header: {token}")
        except KeyError as e:
            token = None
        return token

    def _get_token_from_cookie(self, handler: web.RequestHandler):
        try:
            token = handler.get_secure_cookie(
                WeDPRIdentityProvider.AUTH_TOKEN_FIELD)
        except KeyError as e:
            token = None
        return token

    def _get_token_from_param(self, handler: web.RequestHandler):
        token = None
        try:
            token = handler.get_query_argument(
                WeDPRIdentityProvider.AUTH_TOKEN_FIELD)
            self.log.info(f"#### _get_token_from_param: {token}")
        except Exception as e:
            token = None
        return token

    def _get_token(self, handler: web.RequestHandler):
        token = self._get_token_from_param(handler)
        if token is None:
            token = self._get_token_from_cookie(handler)
        if token is None:
            return self._get_token_from_header(handler)
        return token

    def set_user_cookie(self, handler: web.RequestHandler, token) -> None:
        """Call this on handlers to set the login cookie for success"""
        cookie_options = {}
        cookie_options.update(self.cookie_options)
        cookie_options.setdefault("httponly", True)
        # tornado <4.2 has a bug that considers secure==True as soon as
        # 'secure' kwarg is passed to set_secure_cookie
        secure_cookie = self.secure_cookie
        if secure_cookie is None:
            secure_cookie = handler.request.protocol == "https"
        if secure_cookie:
            cookie_options.setdefault("secure", True)
        # type:ignore[attr-defined]
        cookie_options.setdefault("path", handler.base_url)
        cookie_name = self.get_cookie_name(handler)
        handler.set_secure_cookie(
            WeDPRIdentityProvider.AUTH_TOKEN_FIELD, token, **cookie_options)

    def get_user(self, handler: web.RequestHandler):
        """Authenticate with jwt token, and return the username if login is successful.
        Return None otherwise.
        """
        token = self._get_token(handler)
        if token is None:
            self.log.warning(
                f"WeDPR auth failed for no authorization information defined in the header and cookie! error: {e}")
            return None
        try:
            token_payload = jwt.decode(
                token, self.auth_secret, WeDPRIdentityProvider.AUTH_ALGORITHMS)
            user_info = WeDPRTokenContent.deserialize(token_payload)
        except Exception as e:
            self.log.warning(
                f"WeDPR auth failed for jwt verify failed! error: {e}")
            return None
        if user_info is None:
            return None
        user_name = user_info.get_user_information().username
        if user_name != WeDPRIdentityProvider.CURRENT_USER:
            self.log.warning(
                f"WeDPR auth failed for invalid user, admitted-user: {WeDPRIdentityProvider.CURRENT_USER}, tried user: {user_name}")
            return None
        self.log.info(f"WeDPR auth success, username: {user_name}")
        user = User(username=user_name)
        self.set_user_cookie(handler, token)
        return user



#!/usr/bin/env python
# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.
# -----------------------------------------------------------------------------
# Minimal Python version sanity check (from IPython/Jupyterhub)
# -----------------------------------------------------------------------------

import sys

from setuptools import find_packages, setup
from setuptools.command.bdist_egg import bdist_egg


class bdist_egg_disabled(bdist_egg):
    """Disabled version of bdist_egg

    Prevents setup.py install from performing setuptools' default easy_install,
    which it should never ever do.
    """

    def run(self):
        sys.exit(
            "Aborting implicit building of eggs. Use `pip install .` to install from source."
        )


setup_args = dict(
    name='wedpr-authenticator',
    packages=find_packages(),
    version="3.0.0-20241124",
    description="WeDPR Authenticator: Authenticate JupyterHub users with wedpr providers",
    long_description_content_type="text/markdown",
    author="WeDPR Team",
    author_email="jupyter@googlegroups.com",
    url="https://jupyter.org",
    license="BSD",
    platforms="Linux, Mac OS X, Windows",
    keywords=['Interactive', 'Interpreter', 'Shell', 'Web'],
    python_requires=">=3.8",
    include_package_data=True,
    entry_points={
        'jupyter_server.IdentityProviders': [
            'wedpr-identity-provider = authenticator.wedpr_identity_provider.WeDPRIdentityProvider'
        ],
    },
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
)

setup_args['cmdclass'] = {
    'bdist_egg': bdist_egg if 'bdist_egg' in sys.argv else bdist_egg_disabled,
}

setup_args['install_requires'] = install_requires = []
with open('requirements.txt') as f:
    for line in f.readlines():
        req = line.strip()
        if not req or req.startswith(('-e', '#')):
            continue
        install_requires.append(req)


def main():
    setup(**setup_args)


if __name__ == '__main__':
    main()
